@@ -6,7 +6,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -24,6 +24,7 @@
 #include <gnutls_mpi.h>
 #include <gnutls_errors.h>
 #include <x509_int.h>
+#include <c-ctype.h>
 
 /* Returns 0 if the password is ok, or a negative error
  * code instead.
@@ -35,7 +36,7 @@ _pkcs12_check_pass (const char *pass, size_t plen)
 
   for (i = 0; i < plen; i++)
     {
-      if (isascii (pass[i]))
+      if (c_isascii (pass[i]))
         continue;
       return GNUTLS_E_INVALID_PASSWORD;
     }
@@ -54,7 +55,8 @@ _pkcs12_check_pass (const char *pass, size_t plen)
  * NULL password, and for the password with zero length.
  */
 int
-_gnutls_pkcs12_string_to_key (unsigned int id, const uint8_t * salt,
+_gnutls_pkcs12_string_to_key (gnutls_mac_algorithm_t algo,
+                              unsigned int id, const uint8_t * salt,
                               unsigned int salt_size, unsigned int iter,
                               const char *pw, unsigned int req_keylen,
                               uint8_t * keybuf)
@@ -65,10 +67,11 @@ _gnutls_pkcs12_string_to_key (unsigned int id, const uint8_t * salt,
   bigint_t num_b1 = NULL, num_ij = NULL;
   bigint_t mpi512 = NULL;
   unsigned int pwlen;
-  uint8_t hash[20], buf_b[64], buf_i[MAX_PASS_LEN*2+64], *p;
+  uint8_t hash[MAX_HASH_SIZE], buf_b[64], buf_i[MAX_PASS_LEN*2+64], *p;
   uint8_t d[64];
   size_t cur_keylen;
   size_t n, m, p_size, i_size;
+  unsigned mac_len;
   const uint8_t buf_512[] =      /* 2^64 */
   { 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -128,10 +131,11 @@ _gnutls_pkcs12_string_to_key (unsigned int id, const uint8_t * salt,
     memset (p, 0, p_size);
 
   i_size = 64+p_size;
+  mac_len = _gnutls_hmac_get_algo_len(algo);
 
   for (;;)
     {
-      rc = _gnutls_hash_init (&md, GNUTLS_MAC_SHA1);
+      rc = _gnutls_hash_init (&md, algo);
       if (rc < 0)
         {
           gnutls_assert ();
@@ -143,14 +147,14 @@ _gnutls_pkcs12_string_to_key (unsigned int id, const uint8_t * salt,
       _gnutls_hash_deinit (&md, hash);
       for (i = 1; i < iter; i++)
         {
-          rc = _gnutls_hash_fast (GNUTLS_MAC_SHA1, hash, 20, hash);
+          rc = _gnutls_hash_fast (algo, hash, mac_len, hash);
           if (rc < 0)
             {
               gnutls_assert ();
               goto cleanup;
             }
         }
-      for (i = 0; i < 20 && cur_keylen < req_keylen; i++)
+      for (i = 0; i < mac_len && cur_keylen < req_keylen; i++)
         keybuf[cur_keylen++] = hash[i];
       if (cur_keylen == req_keylen)
         {
@@ -160,7 +164,7 @@ _gnutls_pkcs12_string_to_key (unsigned int id, const uint8_t * salt,
 
       /* need more bytes. */
       for (i = 0; i < 64; i++)
-        buf_b[i] = hash[i % 20];
+        buf_b[i] = hash[i % mac_len];
       n = 64;
       rc = _gnutls_mpi_scan (&num_b1, buf_b, n);
       if (rc < 0)
